# I use some code from talk_AAT.R
library(plotrix)
library(spatstat)
library(TDA)
library(fda)
library(doParallel) 
library(devtools)

# Function to enlarge a circle with radiu=radius, centered at xpos,ypos, by r
fatcircle=function(r,xpos,ypos,radius) {
  angle <- seq(0, 2*pi+0.001, by = 0.001)
  xpos2 <- c(cos(angle) * (radius-r), cos(rev(angle)) * (radius+r))
  ypos2 <- if(r<=radius) {c(sin(angle) * (radius-r), sin(rev(angle)) * (radius+r))} else 
  {c(rep(0,length(angle)), sin(rev(angle)) * (radius+r))}
  polygon(xpos+xpos2, ypos+ypos2, col = 2, border = 2)  
}


############################
### Toy example - Code to obtain Figure 1
############################
rfat=0.5 # In the paper, it corresponds to the parameter t which is 0,  0.5 ,  0.62, and 0.75.

# I plot the circles
radius1=0.5 ; radius2=0.5  ; radius3=0.5
par(mfrow=c(1,1),mar=c(0,0,0,0))
plot(NULL,asp=1,xlim=c(-2.2,2.2),ylim=c(-2,2),ann=F,tcl=0,col.axis=0,axes=F)
draw.circle(x = -1, y = -1,radius =  radius1,lwd=8)
draw.circle(x = 0, y = 1  ,radius =  radius2,lwd=8)
draw.circle(x = 1, y = -1 ,radius =  radius3,lwd=8) 
 
## function to  plot "enlarged" circles
fatcircle(r=rfat,xpos=-1,ypos=-1,0.5) ; fatcircle(r=rfat,xpos=0,ypos=1,0.5) ; fatcircle(r=rfat,xpos=1,ypos=-1,0.5)
draw.circle(x = -1, y = -1,radius =  radius1,lwd=8)
draw.circle(x = 0, y = 1  ,radius =  radius2,lwd=8)
draw.circle(x = 1, y = -1 ,radius =  radius3,lwd=8) 
text(1.58,2.2,labels="t = 0.5",cex=4.9)


###############
## Plot of the corresponding persistent diagram
###############

par(mfrow=c(1,1),mar=c(6.5,7.5,0.2,0.2))
plot(NULL,xlim=c(-0.1,0.9),ylim=c(0,0.9) ,main="", xlab="" ,ylab="",lwd=4,cex.axis=3.5 ,mgp=c(3,2,0),asp=1)
title(ylab="Death time", mgp=c(4.8,1,0) ,cex.lab=4)
title(xlab="Birth time" , mgp=c(5,1,0) ,cex.lab=4)
points(x=0,y=0.5,pch=20,cex=5) ; points(x=0,y=0.62,pch=20,cex=5) 
points(x=0,y=0.5,pch=2,col=2,cex=5,lwd=8) ; points(x=0.62,y=0.75,pch=2,col=2,cex=5,lwd=8)
legend("bottomright",c("k = 0","k = 1"),lty=c(0,0),pch=c(20,2),col=c(1,2),y.intersp = 1.2,lwd=8,cex=4.5,seg.len=1,pt.cex = 4.5)


###############
## Plot of the corresponding APF0 and APF1
###############
 
# Definition of the Accumulated Persistent Function (APF).
# m: the arguments of the APF, Diag: the persistent diagramm, k: the dimension of the topological features
APFtemp=function(m,Diag,k) {
  Diagk=Diag[Diag[,1]==k,]
  if ( is.matrix(Diagk)==TRUE) { lifetime=Diagk[,3]-Diagk[,2] ; meanage= 0.5 * (Diagk[,3]+Diagk[,2] ) ;  sum(lifetime[meanage<=m]) }
  else  { lifetime=Diagk[3]-Diagk[2] ; meanage= 0.5 * (Diagk[3]+Diagk[2] ) ;  sum(lifetime[meanage<=m]) }
}   
APF=Vectorize(APFtemp,vectorize.args = "m","Diag")  

Diag=rbind(c(0,0,0.5),c(0,0,0.62),c(1,0,0.5),c(1,0,0.5),c(1,0,0.5),c(1,0.62,0.75)) #Definition of the persistence diagram corresponding to the toy example.
par(mfrow=c(1,1),mar=c(6.5,8.5,0.2,0.2))
mm=seq(0,1.8,0.001)
plot(mm,APF(mm,Diag,0),xlim=c(0,1),ylim=c(0,1.7) ,main="", xlab="" ,ylab="",lwd=6,cex.axis=3.5 ,mgp=c(3,2,0),type="l")
title(ylab=expression(APF(m)), mgp=c(4.8,1,0) ,cex.lab=4)
title(xlab="m" , mgp=c(5,1,0) ,cex.lab=4)

mmsub=mm[seq(1,length(mm),length.out = 15)] # to plot points on the line and distinguish the curves.
points(mmsub,APF(mmsub,Diag,0),pch=20,cex=5)
mmsub2=mm[seq(1,length(mm),length.out = 9)] # to plot points on the line and distinguish the curves.
lines(mm,APF(mm,Diag,1),lty=2,lwd=8,col=2) ; points(mmsub2,APF(mmsub2,Diag,1),pch=2,cex=4.5,col=2,lwd=6)
legend("bottomright",c(expression(APF[0]),expression(APF[1])),lty=c(1,1)
       ,pch=c(20,2),col=c(1,2),y.intersp = 1.2,lwd=8,cex=4,seg.len=2.4,pt.cex = 4.5)



############################
### brain artery trees dataset - Code to obtain Figure 2
############################

install_github('spatstat/spatstat')  #  development version of the package for better 3d plot.
library(spatstat)

# I use a file tree.csv which is a conversion of a file of the brain artery tree dataset into a .csv file. 
# Each lines represent the coordinate (x,y,z) of a vertex of the tree. 
tree=read.csv(".../tree.csv")
xx=seq(1,length(tree[,1]),by=10)
tree3d=pp3(tree[xx,1],tree[xx,2],1.1*tree[xx,3],
           c(min(tree[,1]),max(tree[,1]) ,min(tree[,2]),max(tree[,2]),min(tree[,3]),max(tree[,3])))
par(mar=c(0,0,0,0)) ; ttheta=5 ; pphi=0
plot(tree3d,cex=0.7,pch=20,col=8, box.back = F, box.front = F,main="",org = c(50,50,50), theta=ttheta, phi=pphi)
thres=40
zz=1.1*tree[xx,3]
tree3d.thres=pp3(tree[xx,1][zz<thres],tree[xx,2][zz<thres],1.1*tree[xx,3][zz<thres],
                 c(min(tree[,1]),max(tree[,1]) ,min(tree[,2]),max(tree[,2]),min(tree[,3]),max(tree[,3])))
plot(tree3d.thres,cex=0.7,col=4,pch=20, box.back = F, box.front = F,main="",org = c(50,50,50), theta=ttheta, phi=pphi, add=T)



############## Plot of the corresponding APF0
library(R.matlab)
### Loading the persistence diagram
CaseInformation.temp=readMat(".../PersistenceDiagrams/CaseInformation.mat")
H0.PD.temp=readMat(".../PersistenceDiagrams/H0PersistenceDiagrams.mat")
H1.PD.temp=readMat(".../H1PersistenceDiagrams.mat")

# I convert into a list of matrices compatible with TDA, each matrices is a PD for TDA
H0.PD.full=NULL ; H1.PD.full=NULL
for (i in (1:length(H0.PD.temp$H0Array))) { H0.PD.full[[i]]= cbind( rep(0,length(H0.PD.temp$H0Array[[i]][[1]][,1]) ) , H0.PD.temp$H0Array[[i]][[1]] )
}
for (i in (1:length(H1.PD.temp$H1Array)) ) { H1.PD.full[[i]]= cbind( rep(1,length(H1.PD.temp$H1Array[[i]][[1]][,1]) ) , H1.PD.temp$H1Array[[i]][[1]] ) 
}
 
PD.brain.full=NULL
for (i in 1:length(H0.PD.full)) { PD.brain.full[[i]]=rbind( rbind(H0.PD.full[[i]],H1.PD.full[[i]]) )}
CaseInformation.full=list( Subject= seq(1,109,1)  ,Ages=CaseInformation.temp$Ages, Sexes=CaseInformation.temp$Sexes )

# #### Empty subject detection, I take them off of the sample after
Emptysubject0=NULL ; Emptysubject1=NULL ; for (i in 1 : 109 ) { Emptysubject0[i]=length(H0.PD.full[[i]]) }
Emptysubject0=which(Emptysubject0==0) ;  Emptysubject1=which(Emptysubject1==0) 
H0.PD=H0.PD.full[-Emptysubject0] ; H1.PD=H1.PD.full[-Emptysubject0]
# #### Some death times are at -1. I erease them. They represent topological features that never die (see the article for more details).
for (i in (1:length(H1.PD)) ) {  H1.PD[[i]][which(H1.PD[[i]][,3]==-1) ,3] = Inf  }

mm0=seq(0,160,0.005) 
registerDoParallel(cores=6)
APF0.brain=foreach(i=1:length(H0.PD), .combine=rbind) %dopar% { APF(mm0,H0.PD[[i]],0) }

 
####### Plot of the corresponding persistent diagrams
par(mfrow=c(1,1),mar=c(6.5,7.5,0.2,0.2))
plot(H0.PD[[2]][,2],H0.PD[[2]][,3],xlim=c(-0.1,95),ylim=c(0,90),cex=2 ,main="", xlab="" ,ylab="",lwd=4,cex.axis=3.5 ,mgp=c(3,2,0),asp=1)
title(ylab="Death time", mgp=c(4.8,1,0) ,cex.lab=4)
title(xlab="Birth time" , mgp=c(5,1,0) ,cex.lab=4)
 
plot(H1.PD[[2]][,2],H1.PD[[2]][,3],xlim=c(-0.1,25),ylim=c(0,25),cex=2 ,col=2,pch=2 ,main="", xlab="" ,ylab="",lwd=4,cex.axis=3.5 ,mgp=c(3,2,0),asp=1)
title(ylab="Death time", mgp=c(4.8,1,0) ,cex.lab=4)
title(xlab="Birth time" , mgp=c(5,1,0) ,cex.lab=4)


