library(spatstat)
library(plotrix) #package to draw circles easily
library(doParallel)
library(TDA) #package to compute  Delaunay-complex filtrations
library(depthTools) #package that contains the method by Lopez-Pintado et al. 2010


# Modification of the TDA function to be compatible with ppp objects from the R-package spatstat
# and to return a matrix in place of object of class diagram
alphaDiag=function(PP){ 
  if (is.ppp(PP)==TRUE ) {
    sim_PP_coord=cbind( PP$x, PP$y ) 
  } else {
    sim_PP_coord=cbind(PP[,1], PP[,2] ) 
  }
  Diag=cbind(alphaComplexDiag(sim_PP_coord)$diagram[,1],sqrt(alphaComplexDiag(sim_PP_coord)$diagram[,2]),sqrt(alphaComplexDiag(sim_PP_coord)$diagram[,3]))
  colnames(Diag)=c("dimension","Birth","Death")
  return(Diag)
}

# Definition of the Accumulated Persistent Function (APF).
# m: the arguments of the APF, Diag: the persistent diagramm, k: the dimension of the topological features
APFtemp=function(m,Diag,k) {
  Diagk=Diag[Diag[,1]==k,]
  if ( is.matrix(Diagk)==TRUE) { lifetime=Diagk[,3]-Diagk[,2] ; meanage= 0.5 * (Diagk[,3]+Diagk[,2] ) ;  sum(lifetime[meanage<=m]) }
  else  { lifetime=Diagk[3]-Diagk[2] ; meanage= 0.5 * (Diagk[3]+Diagk[2] ) ;  sum(lifetime[meanage<=m]) }
} 
APF=Vectorize(APFtemp,vectorize.args = "m","Diag")  


mm=seq(0,1,0.0025) ; trained=1:50
N=150 
# Noise amplitude
nbrepoint=100
noiseamp = 0.1

registerDoParallel(cores=3)
P1=NULL ; P2=NULL
RADIUS=0.8  # Radius of the smallest circle. Choose as 0.8 and 0.9 in Example 10.

# Function to compute the percentage of wrongly assigned APF0
error.supervised.H0= foreach( ii=1:500 , .combine=rbind) %dopar% {
  for (i in 1:5) {
    repartition=sample(c(1,2),nbrepoint,replace=T)
    P1[[i]] =  two.unif.circle(nbrepoint1 = length(repartition[repartition==1]) 
                               , nbrepoint2 = length(repartition[repartition==2]) ,r1=1,r2=0.5,c1=0,c2=1.5) 
    + cbind( rnorm(nbrepoint, 0, noiseamp )  ,  rnorm(nbrepoint, 0, noiseamp  )  )
    repartition=sample(c(1,2),nbrepoint,replace=T)
    P2[[i]] =  two.unif.circle(nbrepoint1 = length(repartition[repartition==1]) 
                               , nbrepoint2 = length(repartition[repartition==2]) ,r1=RADIUS,r2=0.5,c1=0,c2=1.5) 
    + cbind( rnorm(nbrepoint, 0, noiseamp )  ,  rnorm(nbrepoint, 0, noiseamp  )  )
  }
  for (i in 6:N) {
    P1[[i]] =   circleUnif(nbrepoint, r = 1) +   cbind( rnorm(nbrepoint, 0, noiseamp )  ,  rnorm(nbrepoint, 0, noiseamp  )  )
    P2[[i]] = circleUnif(nbrepoint, r = RADIUS) +  cbind( rnorm(nbrepoint, 0, noiseamp )  ,  rnorm(nbrepoint, 0, noiseamp  )  )
  }
  DiagP1 =  foreach(ii=1:N) %do% { alphaDiag(P1[[ii]]) }
  P1_APF0= foreach(ii=1:N, .combine=rbind) %do% { APF( mm, Diag=DiagP1[[ii]], k=0 )  }
  
  DiagP2 =  foreach(ii=1:N) %do% { alphaDiag(P2[[ii]]) }
  P2_APF0= foreach(ii=1:N, .combine=rbind) %do% { APF( mm, Diag=DiagP2[[ii]], k=0 )  }
  
  xl0 = rbind(P1_APF0[trained,], P2_APF0[trained,])
  yl0 = c( rep(1,length(trained))  , rep(2,length(trained))  )
  xt0 = rbind(P1_APF0[-trained,] ,P2_APF0[-trained,])
  
  DS0=classDS(xl0,yl0, xt0, alpha=0.2)
  
  c( length(which(DS0[1:100]!=1))/(N-length(trained)), length(which(DS0[101:200]!=2))/ (N-length(trained))) 
}
100*mean((error.supervised.H0[,1]+error.supervised.H0[,2])/2) # Display in percentage the mean of the number of wrongly assigned APF0
100*sd((error.supervised.H0[,1]+error.supervised.H0[,2])/2)  # Display in percentage the standard deviation of the number of wrongly assigned APF0

# Function to compute the percentage of wrongly assigned APF0
error.supervised.H1= foreach( ii=1:500 , .combine=rbind) %dopar% {
  
  
  for (i in 1:5) {
    repartition=sample(c(1,2),nbrepoint,replace=T)
    P1[[i]] =  two.unif.circle(nbrepoint1 = length(repartition[repartition==1]) 
                               , nbrepoint2 = length(repartition[repartition==2]) ,r1=1,r2=0.5,c1=0,c2=1.5) 
    + cbind( rnorm(nbrepoint, 0, noiseamp )  ,  rnorm(nbrepoint, 0, noiseamp  )  )
    repartition=sample(c(1,2),nbrepoint,replace=T)
    P2[[i]] =  two.unif.circle(nbrepoint1 = length(repartition[repartition==1]) 
                               , nbrepoint2 = length(repartition[repartition==2]) ,r1=RADIUS,r2=0.5,c1=0,c2=1.5) 
    + cbind( rnorm(nbrepoint, 0, noiseamp )  ,  rnorm(nbrepoint, 0, noiseamp  )  )
  }
  for (i in 6:N) {
    P1[[i]] =   circleUnif(nbrepoint, r = 1) +   cbind( rnorm(nbrepoint, 0, noiseamp )  ,  rnorm(nbrepoint, 0, noiseamp  )  )
    P2[[i]] = circleUnif(nbrepoint, r = RADIUS) +  cbind( rnorm(nbrepoint, 0, noiseamp )  ,  rnorm(nbrepoint, 0, noiseamp  )  )
  }
  
  DiagP1 =  foreach(ii=1:N) %do% { alphaDiag(P1[[ii]]) }
  P1_APF1= foreach(ii=1:N, .combine=rbind) %do% { APF( mm, Diag=DiagP1[[ii]], k=1 )  }
  
  DiagP2 =  foreach(ii=1:N) %do% { alphaDiag(P2[[ii]]) }
  P2_APF1= foreach(ii=1:N, .combine=rbind) %do% { APF( mm, Diag=DiagP2[[ii]], k=1 )  }
  
  xl1 = rbind(P1_APF1[trained,] ,P2_APF1[trained,])
  yl1 = c( rep(1,length(trained)),   rep(2,length(trained))  )
  xt1 = rbind(P1_APF1[-trained,]  ,P2_APF1[-trained,])
  
  DS1=classDS(xl1,yl1, xt1, alpha=0.2)
  
  c( length(which(DS1[1:100]!=1))/(N-length(trained)), length(which(DS1[101:200]!=2))/ (N-length(trained))) 
  
}
100*mean((error.supervised.H1[,1]+error.supervised.H1[,2])/2) # Display in percentage the mean of the number of wrongly assigned APF1
100*sd((error.supervised.H1[,1]+error.supervised.H1[,2])/2)  # Display in percentage the standard deviation of the number of wrongly assigned APF1




#Code for figure 15 Top panels

# simulation of the points patterns
for (i in 1:5) {
  repartition=sample(c(1,2),nbrepoint,replace=T)
  P1[[i]] =  two.unif.circle(nbrepoint1 = length(repartition[repartition==1]) 
                             , nbrepoint2 = length(repartition[repartition==2]) ,r1=1,r2=0.5,c1=0,c2=1.5) 
  + cbind( rnorm(nbrepoint, 0, noiseamp )  ,  rnorm(nbrepoint, 0, noiseamp  )  )
  repartition=sample(c(1,2),nbrepoint,replace=T)
  P2[[i]] =  two.unif.circle(nbrepoint1 = length(repartition[repartition==1]) 
                             , nbrepoint2 = length(repartition[repartition==2]) ,r1=RADIUS,r2=0.5,c1=0,c2=1.5) 
  + cbind( rnorm(nbrepoint, 0, noiseamp )  ,  rnorm(nbrepoint, 0, noiseamp  )  )
}
for (i in 6:N) {
  P1[[i]] =   circleUnif(nbrepoint, r = 1) +   cbind( rnorm(nbrepoint, 0, noiseamp )  ,  rnorm(nbrepoint, 0, noiseamp  )  )
  P2[[i]] = circleUnif(nbrepoint, r = RADIUS) +  cbind( rnorm(nbrepoint, 0, noiseamp )  ,  rnorm(nbrepoint, 0, noiseamp  )  )
}
# Computation of the persistence diagrams and APFs  
DiagP1 =  foreach(ii=1:N) %do% { alphaDiag(P1[[ii]]) }
P1_APF0= foreach(ii=1:N, .combine=rbind) %do% { APF( mm, Diag=DiagP1[[ii]], k=0 )  }
P1_APF1= foreach(ii=1:N, .combine=rbind) %do% { APF( mm, Diag=DiagP1[[ii]], k=1 )  }
DiagP2 =  foreach(ii=1:N) %do% { alphaDiag(P2[[ii]]) }
P2_APF0= foreach(ii=1:N, .combine=rbind) %do% { APF( mm, Diag=DiagP2[[ii]], k=0 )  }
P2_APF1= foreach(ii=1:N, .combine=rbind) %do% { APF( mm, Diag=DiagP2[[ii]], k=1 )  }

 



par(mar=c(7,7,0.5,0.5))
plot(mm, P1_APF0[trained[1],] ,type="l" ,ylim=c(0,6),xlim=c(0,0.32),col=0,main="", xlab="" ,ylab="",cex.axis=2.5,mgp=c(3,2,0) )
title(ylab=expression(), mgp=c(4,1,0) ,cex.lab=3)
title(xlab=expression("m") , mgp=c(5,1,0) ,cex.lab=3)
lines(mm, tmean(P1_APF0[trained,])$tm,type="l",col=2,lwd=6,lty=1)
lines(mm, tmean(P2_APF0[trained,])$tm,type="l",col=4 ,lwd=6,lty=2)
legend("bottomright",c(expression(bar(A)[1]^0.2),expression(bar(A)[2]^0.2) )
       ,col=c("2","4"),y.intersp = 1.5,lwd=6,cex=3,seg.len=3,lty=c(1,2))

plot(mm, P1_APF1[trained[1],] ,type="l" ,ylim=c(0,0.9),xlim=c(0,1),col=0,main="", xlab="" ,ylab="",cex.axis=2.5,mgp=c(3,2,0) )
title(ylab=expression(), mgp=c(4,1,0) ,cex.lab=3)
title(xlab=expression("m") , mgp=c(5,1,0) ,cex.lab=3)
lines(mm, tmean(P1_APF1[trained,])$tm,type="l",col=2,lwd=6,lty=1)
lines(mm, tmean(P2_APF1[trained,])$tm,type="l",col=4 ,lwd=6,lty=2)
legend("bottomright",c(expression(bar(A)[1]^0.2),expression(bar(A)[2]^0.2) )
       ,col=c("2","4"),y.intersp = 1.5,lwd=6,cex=3,seg.len=3,lty=c(1,2))


#Code for figure 15 Bottom panels

# simulation of the points patterns
  
  for (i in 1:5) {
    repartition=sample(c(1,2),nbrepoint,replace=T)
    P1[[i]] =  two.unif.circle(nbrepoint1 = length(repartition[repartition==1]) 
                               , nbrepoint2 = length(repartition[repartition==2]) ,r1=1,r2=0.5,c1=0,c2=1.5) 
    + cbind( rnorm(nbrepoint, 0, noiseamp )  ,  rnorm(nbrepoint, 0, noiseamp  )  )
    repartition=sample(c(1,2),nbrepoint,replace=T)
    P2[[i]] =  two.unif.circle(nbrepoint1 = length(repartition[repartition==1]) 
                               , nbrepoint2 = length(repartition[repartition==2]) ,r1=0.8,r2=0.5,c1=0,c2=1.5) 
    + cbind( rnorm(nbrepoint, 0, noiseamp )  ,  rnorm(nbrepoint, 0, noiseamp  )  )
  }
  for (i in 6:N) {
    P1[[i]] =   circleUnif(nbrepoint, r = 1) +   cbind( rnorm(nbrepoint, 0, noiseamp )  ,  rnorm(nbrepoint, 0, noiseamp  )  )
    P2[[i]] = circleUnif(nbrepoint, r = 0.8) +  cbind( rnorm(nbrepoint, 0, noiseamp )  ,  rnorm(nbrepoint, 0, noiseamp  )  )
  }

# Computation of the persistence diagrams and APF0s  
DiagP1 =  foreach(ii=1:N) %do% { alphaDiag(P1[[ii]]) }
P1_APF0= foreach(ii=1:N, .combine=rbind) %do% { APF( mm, Diag=DiagP1[[ii]], k=0 )  }
DiagP2 =  foreach(ii=1:N) %do% { alphaDiag(P2[[ii]]) }
P2_APF0= foreach(ii=1:N, .combine=rbind) %do% { APF( mm, Diag=DiagP2[[ii]], k=0 )  }

# Supervised classification
xl0 = rbind(P1_APF0[trained,], P2_APF0[trained,])
yl0 = c( rep(1,length(trained))  , rep(2,length(trained))  )
xt0 = rbind(P1_APF0[-trained,] ,P2_APF0[-trained,])
DS0=classDS(xl0,yl0, xt0, alpha=0.2)

# Plot of the bottom panels
par(mar=c(0,0,0,0))
wrong1=which(DS0[1:100]==2)  ; wrong2=which(DS0[101:200]==1)   # Determine the curves wrongly assigned
plot(P1[[wrong1[1]]],cex=2.5,pch=20,col=2,asp=1,xlim=c(-1.2,1.2),ylim=c(-1.2,1.2),xlab="",ylab="",main="",axes=F)
draw.circle(0,0,radius=1,lwd=2)
draw.circle(0,0,radius=0.8,lwd=2)

plot(P2[[wrong2[3]]],cex=2.5,pch=20,col=4,asp=1,xlim=c(-1.2,1.2),ylim=c(-1.2,1.2),xlab="",ylab="",main="",axes=F)
draw.circle(0,0,radius=1,lwd=2)
draw.circle(0,0,radius=0.8,lwd=2)


