library(plotrix) #package to draw circles easily
library(parallel)
library(TDA) #package to compute  Delaunay-complex filtrations
library(fda) #to compute the functionnal boxplots
library(depthTools) 
library(spatstat)

# Modification of the TDA function to be compatible with ppp objects from the R-package spatstat
# and to return a matrix in place of object of class diagram
alphaDiag=function(PP){ 
  if (is.ppp(PP)==TRUE ) {
    sim_PP_coord=cbind( PP$x, PP$y ) 
  } else {
    sim_PP_coord=cbind(PP[,1], PP[,2] ) 
  }
  Diag=cbind(alphaComplexDiag(sim_PP_coord)$diagram[,1],sqrt(alphaComplexDiag(sim_PP_coord)$diagram[,2]),sqrt(alphaComplexDiag(sim_PP_coord)$diagram[,3]))
  colnames(Diag)=c("dimension","Birth","Death")
  return(Diag)
}

# Definition of the Accumulated Persistent Function (APF).
# m: the arguments of the APF, Diag: the persistent diagramm, k: the dimension of the topological features
APFtemp=function(m,Diag,k) {
  Diagk=Diag[Diag[,1]==k,]
  if ( is.matrix(Diagk)==TRUE) { lifetime=Diagk[,3]-Diagk[,2] ; meanage= 0.5 * (Diagk[,3]+Diagk[,2] ) ;  sum(lifetime[meanage<=m]) }
  else  { lifetime=Diagk[3]-Diagk[2] ; meanage= 0.5 * (Diagk[3]+Diagk[2] ) ;  sum(lifetime[meanage<=m]) }
} 
APF=Vectorize(APFtemp,vectorize.args = "m","Diag")  


# Modification of the fbplot function from the R-package fda in order to have more options on the plot (color, line width ... etc)
# The first three function are used in the last one, fbplotperso
combinatperso=function(n,p){
  if (n<p){combinat=0}
  else {combinat=exp(lfactorial(n)-(lfactorial(p)+lfactorial(n-p)))}
}
fBD2perso=function(data){
  p=dim(data)[1]
  n=dim(data)[2]
  rmat=apply(data,1,rank)
  down=apply(rmat,1,min)-1
  up=n-apply(rmat,1,max)
  (up*down+n-1)/combinatperso(n,2)
  
}
fMBDperso=function(data){
  p=dim(data)[1]
  n=dim(data)[2]
  rmat=apply(data,1,rank)
  down=rmat-1
  up=n-rmat
  (rowSums(up*down)/p+n-1)/combinatperso(n,2)
}
fbplotperso=function (fit, x = NULL, method = "MBD", depth = NULL, plot = TRUE, lwdenv=2 ,colbordenv=4,colenv=6 ,colbornes=4, ltybornes=1 , lwdbornes=2,prob = 0.5,  lwdmeancurve=2   ,outliercol = 2, fullout = FALSE, fact = 1.5, xlim = c(1, nrow(fit)), ylim = c(min(fit) -  0.5 * diff(range(fit)), max(fit) + 0.5 * diff(range(fit))),  ...) {
  tp = dim(fit)[1]
  n = dim(fit)[2]
  if (length(x) == 0) {x = 1:tp}
  # if (length(depth) != length(fit[1,])) { return(" error : depth must be calculated correctly ") }
  if (length(depth)==0){
    if (method=='BD2') {depth=fBD2perso(fit)}
    else if (method=='MBD') {depth=fMBDperso(fit)}
    else if (method=='Both') {depth=round(fBD2perso(fit),4)*10000+fMBDperso(fit)} }
  dp_s = sort(depth, decreasing = TRUE)
  index = order(depth, decreasing = TRUE)
  med = depth == max(depth)
  medavg = matrix(fit[, med], ncol = sum(med), nrow = tp)
  y = apply(medavg, 1, mean)
  if (plot) {
    plot(x, y, lty = 1, lwd = 2, col = 1, type = "l", xlim, 
         ylim, ...)
  }
  xx=NULL ; yy=NULL
  for (pp in 1:length(prob)) {
    m = ceiling(n * prob[pp])
    center = fit[, index[1:m]]
    out = fit[, index[(m + 1):n]]
    inff = apply(center, 1, min)
    supp = apply(center, 1, max)
    
    
    #  Begin Outliers detection 
    if (prob[pp] == 0.5) {
      dist = fact * (supp - inff)
      upper = supp + dist
      lower = inff - dist
      outly = (fit < lower) + (fit > upper)
      outcol = colSums(outly)
      remove = (outcol > 0)
      colum = 1:n
      outpoint = colum[remove == 1]
      out = fit[, remove]
      woout = fit
      good = woout[, (remove == 0), drop = FALSE]
      maxcurve = apply(good, 1, max)
      mincurve = apply(good, 1, min)
      if (sum(outly) > 0) {
        if (plot) {
          matlines(x, out, lty = 2, col = outliercol, type = "l", ...)
        }
      }
      barval = (x[1] + x[tp])/2
      bar = which(sort(c(x, barval)) == barval)[1]
      if (plot) {
        # Code unused in the article
        # lines(c(x[bar], x[bar]), c(maxcurve[bar], supp[bar]), col = barcol, lwd = 2)
        # lines(c(x[bar], x[bar]), c(mincurve[bar], inff[bar]), col = barcol, lwd = 2)
      }
    }
    # End outliers detection
    
    # Begin plot the region of the alpha%  most central curves
    xx[[pp]] = c(x, x[order(x, decreasing = TRUE)])
    suppinv = supp[order(x, decreasing = TRUE)]
    yy[[pp]] = c(inff, suppinv)
    if (plot) {
      if (prob[pp] == 0.5) {
        polygon(xx[[pp]], yy[[pp]], col = colenv, border = colbordenv, 
                lwd = lwdenv)
      }
      else {
        polygon(xx[[pp]], yy[[pp]], col = colenv, border = NA)
      }
    }
    # End plot the region of the alpha%  most central curves
    
  }
  if (plot) {
    lines(x, fit[, index[1]], lty = 1, lwd = lwdmeancurve, col = 1, type = "l")
    lines(x, maxcurve, col = colbornes, lwd = lwdbornes, lty=ltybornes)
    lines(x, mincurve, col = colbornes, lwd = lwdbornes, lty=ltybornes)
    if (fullout) {
      if (sum(outly) > 0) {
        if (plot) {
          matlines(x, out, lty = 2, col = outliercol, type = "l")
        }
      }
    }
  }
  return(list(outpoint = outpoint, medcurve = which(med), envXX=xx,envYY=yy,maxcurve=maxcurve,mincurve=mincurve ))
}

# function to plot points on two or three circles
two.unif.circle=function(nbrepoint1,nbrepoint2,r1,r2,c1,c2){
  if ( length(c1)==1 ) {  circle1= c1 + circleUnif(nbrepoint1, r = r1) } else {circle1 = cbind( rep(c1[1],nbrepoint1) ,   rep(c1[2],nbrepoint1)) + circleUnif(nbrepoint1, r = r1) }
  if ( length(c2)==1 ) {  circle2= c2 + circleUnif(nbrepoint2, r = r2) } else {circle2 = cbind( rep(c2[1],nbrepoint2) ,   rep(c2[2],nbrepoint2)) + circleUnif(nbrepoint2, r = r2) }
  return(rbind(circle1,circle2))
}



mm=seq(0,2,0.001) # Interval considered
N=50 # size of the sample

# Simulation of the different samples
P1=NULL ; P2 = NULL ; P3 = NULL ; P3=NULL ; P4=NULL
smallradius=0.7 # parameter for the smallest circles
noiseamp = 0.1 #amplitude of the deviation
nbrepoint= 100 # number of points for each point pattern
for (i in 1:N) {
  P1[[i]] = circleUnif(nbrepoint, r = 1) + cbind( rnorm(nbrepoint, 0, noiseamp )  ,  rnorm(nbrepoint, 0, noiseamp  )  )
  P2[[i]] = rbind ( cbind( rnorm(nbrepoint/2,0,0.2), rnorm(nbrepoint/2,0,0.2) ) ,  cbind( 1+rnorm(nbrepoint/2,0,0.2), 0.5+rnorm(nbrepoint/2,0,0.2) ) )
  repartition=sample(c(1,2), nbrepoint,replace=T)
  P3[[i]] =   two.unif.circle(nbrepoint1 = length(repartition[repartition==1])  
                              , nbrepoint2 = length(repartition[repartition==2])  ,r1=1,r2=0.5,c1=-1,c2=1) 
  + cbind( rnorm(nbrepoint, 0, noiseamp )  ,  rnorm(nbrepoint, 0, noiseamp  )  )
  P4[[i]] = circleUnif(nbrepoint, r = smallradius) + cbind( rnorm(nbrepoint, 0, noiseamp)  ,  rnorm(nbrepoint, 0, noiseamp) )
}   

# Computation of the persistent diagrams
registerDoParallel(cores = 3)
DiagP1 =  foreach(ii=1:N) %dopar% { alphaDiag(P1[[ii]]) }
DiagP2 =  foreach(ii=1:N) %dopar% { alphaDiag(P2[[ii]]) }
DiagP3 =  foreach(ii=1:N) %dopar% { alphaDiag(P3[[ii]]) }
DiagP4 =  foreach(ii=1:N) %dopar% { alphaDiag(P4[[ii]]) }

# Computation of the matrix required by the functions fbplot and fbplotperso
# Each line of the matrix represent an APF
MatP1_APF0=foreach(ii=1:N, .combine=rbind) %dopar% { APF( mm, Diag=DiagP1[[ii]], k=0 )  } 
MatP1_APF1=foreach(ii=1:N, .combine=rbind) %dopar% { APF( mm, Diag=DiagP1[[ii]], k=1 )  }
MatP2_APF0=foreach(ii=1:N, .combine=rbind) %dopar% { APF( mm, Diag=DiagP2[[ii]], k=0 )  }
MatP2_APF1=foreach(ii=1:N, .combine=rbind) %dopar% { APF( mm, Diag=DiagP2[[ii]], k=1 )  }
MatP3_APF0=foreach(ii=1:N, .combine=rbind) %dopar% { APF( mm, Diag=DiagP3[[ii]], k=0 )  }
MatP3_APF1=foreach(ii=1:N, .combine=rbind) %dopar% { APF( mm, Diag=DiagP3[[ii]], k=1 )  } 
MatP4_APF0=foreach(ii=1:N, .combine=rbind) %dopar% { APF( mm, Diag=DiagP4[[ii]], k=0 )  }
MatP4_APF1=foreach(ii=1:N, .combine=rbind) %dopar% { APF( mm, Diag=DiagP4[[ii]], k=1 )  }


# Code for Figure 11 Left panel
par(mar=c(7,7,0.5,0.5))
testoutliers0=rbind(MatP1_APF0 , MatP2_APF0[1:5,] , MatP3_APF0[1:5,] ,MatP4_APF0[1:5,])
depthtestoutliers0=MBD( testoutliers0,plotting=F)$MBD
FB_outliers0=fbplotperso(t(testoutliers0), x = mm, depth = depthtestoutliers0, method = "MBD", fact=1.5, lwdbornes = 4,
                         prob = 0.5, outliercol = 0,xlim=c(0,0.8) , 
                         ylim=c(0,9),xlab="",ylab="",cex.lab=2,cex.axis=2.5 ,mgp=c(3,2,0) )
title(ylab=expression(APF[0](m)), mgp=c(4,1,0) ,cex.lab=3)
title(xlab=expression("m") , mgp=c(5,1,0) ,cex.lab=3)
FB_outliers0$outpoint
for (i in  FB_outliers0$outpoint[FB_outliers0$outpoint >= 51 & FB_outliers0$outpoint<=55] ) { lines(mm,testoutliers0[i,],col=2,lwd=2)  }
for (i in  FB_outliers0$outpoint[FB_outliers0$outpoint >= 56 & FB_outliers0$outpoint<=60] ) { lines(mm,testoutliers0[i,],col=5,lwd=2)  }
for (i in  FB_outliers0$outpoint[FB_outliers0$outpoint >= 61 & FB_outliers0$outpoint<=65] ) { lines(mm,testoutliers0[i,],col=3,lwd=2)  }
legend("topleft",c("Gaussian mixture","Two circles","Circle r=0.7"),col=c("2","5","3") ,y.intersp = 1.5,lwd=4,cex=1.9,seg.len=3)


# Code for Figure 11 Right panel
par(mar=c(7,7,0.5,0.5))
testoutliers1=rbind(MatP1_APF1 , MatP2_APF1[1:5,] , MatP3_APF1[1:5,] ,MatP4_APF1[1:5,])
depthtestoutliers1=MBD( testoutliers1,plotting=F)$MBD
FB_outliers1=fbplotperso(t(testoutliers1), x = mm,colenv=6, colbornes = 4 , ltybornes= 1, lwdbornes = 4 ,
                         lwdenv = 2 , colbordenv = 4, lwdmeancurve = 2 ,
                         depth = depthtestoutliers1, method = "MBD", fact=1.5, prob = 0.5,
                         outliercol = 0,xlim=c(0,1.5) , ylim=c(0,1.5),xlab="",ylab=""
                         ,cex.lab=2 ,cex.lab=2,cex.axis=2.5 ,mgp=c(3,2,0) )
title(ylab=expression(APF[1](m)), mgp=c(4,1,0) ,cex.lab=3)
title(xlab=expression("m") , mgp=c(5,1,0) ,cex.lab=3)
FB_outliers1$outpoint
for (i in  FB_outliers1$outpoint[FB_outliers1$outpoint >= 51 & FB_outliers1$outpoint<=55] ) { lines(mm,testoutliers1[i,],col=2,lwd=2)  }
for (i in  FB_outliers1$outpoint[FB_outliers1$outpoint >= 56 & FB_outliers1$outpoint<=60] ) { lines(mm,testoutliers1[i,],col=5,lwd=2)  }
for (i in  FB_outliers1$outpoint[FB_outliers1$outpoint >= 61 & FB_outliers1$outpoint<=65] ) { lines(mm,testoutliers1[i,],col=3,lwd=2)  }
legend("topleft",c("Gaussian mixture","Two circles","Circle r=0.7"),col=c("2","5","3"),y.intersp = 1.5,lwd=4,cex=1.9,seg.len=3)




