library(plotrix) #package to draw circles easily
library(doParallel)
library(TDA) #package to compute  Delaunay-complex filtrations


# Modification of the TDA function to be compatible with ppp objects from the R-package spatstat
# and to return a matrix in place of object of class diagram
alphaDiag=function(PP){ 
  if (is.ppp(PP)==TRUE ) {
    sim_PP_coord=cbind( PP$x, PP$y ) 
  } else {
    sim_PP_coord=cbind(PP[,1], PP[,2] ) 
  }
  Diag=cbind(alphaComplexDiag(sim_PP_coord)$diagram[,1],sqrt(alphaComplexDiag(sim_PP_coord)$diagram[,2]),sqrt(alphaComplexDiag(sim_PP_coord)$diagram[,3]))
  colnames(Diag)=c("dimension","Birth","Death")
  return(Diag)
}

# Definition of the Accumulated Persistent Function (APF).
# m: the arguments of the APF, Diag: the persistent diagramm, k: the dimension of the topological features
APFtemp=function(m,Diag,k) {
  Diagk=Diag[Diag[,1]==k,]
  if ( is.matrix(Diagk)==TRUE) { lifetime=Diagk[,3]-Diagk[,2] ; meanage= 0.5 * (Diagk[,3]+Diagk[,2] ) ;  sum(lifetime[meanage<=m]) }
  else  { lifetime=Diagk[3]-Diagk[2] ; meanage= 0.5 * (Diagk[3]+Diagk[2] ) ;  sum(lifetime[meanage<=m]) }
} 
APF=Vectorize(APFtemp,vectorize.args = "m","Diag")  

# function to plot point on two circles
two.unif.circle=function(nbrepoint1,nbrepoint2,r1,r2,c1,c2){
  if ( length(c1)==1 ) {  circle1= c1 + circleUnif(nbrepoint1, r = r1) } else {circle1 = cbind( rep(c1[1],nbrepoint1) ,   rep(c1[2],nbrepoint1)) + circleUnif(nbrepoint1, r = r1) }
  if ( length(c2)==1 ) {  circle2= c2 + circleUnif(nbrepoint2, r = r2) } else {circle2 = cbind( rep(c2[1],nbrepoint2) ,   rep(c2[2],nbrepoint2)) + circleUnif(nbrepoint2, r = r2) }
  return(rbind(circle1,circle2))
}


mm=seq(0,0.5,length.out = 2500)

N=50 # size of each sample
P1=NULL ;  P2=NULL ;P3=NULL 

# Function to compute the performance  of the K-means algorithm in Example 9
# It return the number of curve wrongly assigned
# nbrepoint: number of point in each point pattern, kk: dimension of the considered topological feature
# RADIUS: radius of circle in the third sample. It is 0.8  in the article.
perf.kmeans=function(nbrepoint,kk,RADIUS) { 
  for (i in 1:N) {
    P1[[i]] =   circleUnif(nbrepoint, r = 1)  + cbind( rnorm(nbrepoint, 0, noiseamp )  ,  rnorm(nbrepoint, 0, noiseamp  )  )
    repartition=sample(c(1,2),nbrepoint,replace=T)
    P2[[i]] =  two.unif.circle(nbrepoint1 = length(repartition[repartition==1]) 
                               , nbrepoint2 = length(repartition[repartition==2]) ,r1=0.5,r2=0.5,c1=-1,c2=1) 
    + cbind( rnorm(nbrepoint, 0, noiseamp )  ,  rnorm(nbrepoint, 0, noiseamp  )  )
    P3[[i]] = circleUnif(nbrepoint, r = RADIUS)  + cbind( rnorm(nbrepoint, 0, noiseamp )  ,  rnorm(nbrepoint, 0, noiseamp  )  )
  }  
  
  DiagP1 =  foreach(ii=1:N) %do% { alphaDiag(P1[[ii]]) }
  P1_APF= foreach(ii=1:N, .combine=rbind) %do% { APF( mm, Diag=DiagP1[[ii]], k=kk )  }
  DiagP2 =  foreach(ii=1:N) %do% { alphaDiag(P2[[ii]]) }
  P2_APF= foreach(ii=1:N, .combine=rbind) %do% { APF( mm, Diag=DiagP2[[ii]], k=kk )  }
  DiagP3 =  foreach(ii=1:N) %do% { alphaDiag(P3[[ii]]) }
  P3_APF= foreach(ii=1:N, .combine=rbind) %do% { APF( mm, Diag=DiagP3[[ii]], k=kk )  }
  fun.curves=rbind(P1_APF,P2_APF,P3_APF)
  test=kmeans(fun.curves,centers=3,iter.max = 100,nstart = 200 )
  error.temp=c( min(table(test$cluster[1:50])), min(table(test$cluster[51:100])), min(table(test$cluster[101:150])) )
  return(sum(replace(error.temp,error.temp == 50, 0) ))
} 

# number of points and noise amplitude
nbrepoint=100 ; noiseamp = 0.1

registerDoParallel(cores=3)

error.kmeans.H0.2=foreach( jj = 1:500, .combine=rbind) %dopar% { perf.kmeans(100,kk=0, RADIUS = 0.8) } 
mean(error.kmeans.H0.2/150); sd(error.kmeans.H0.2/150) # display the mean and standard deviation of wrongly assigned APF0
error.kmeans.H1.1=foreach( jj = 1:500, .combine=rbind) %dopar% { perf.kmeans(100,kk=1, RADIUS = 0.8) }
mean(error.kmeans.H1.1/150); sd(error.kmeans.H1.1/150)  # display the mean and standard deviation of wrongly assigned APF1


#Code for Figure 14
for (i in 1:N) {
  P1[[i]] =   circleUnif(nbrepoint, r = 1)  + cbind( rnorm(nbrepoint, 0, noiseamp )  ,  rnorm(nbrepoint, 0, noiseamp  )  )
  repartition=sample(c(1,2),nbrepoint,replace=T)
  P2[[i]] =  two.unif.circle(nbrepoint1 = length(repartition[repartition==1]) 
                             , nbrepoint2 = length(repartition[repartition==2]) ,r1=0.5,r2=0.5,c1=-1,c2=1) 
  + cbind( rnorm(nbrepoint, 0, noiseamp )  ,  rnorm(nbrepoint, 0, noiseamp  )  )
  P3[[i]] = circleUnif(nbrepoint, r = 0.8)  + cbind( rnorm(nbrepoint, 0, noiseamp )  ,  rnorm(nbrepoint, 0, noiseamp  )  )
}  


mm=seq(0,0.5,length.out = 2500) ; registerDoParallel(cores=4)
DiagP1 =  foreach(ii=1:N) %dopar% { alphaDiag(P1[[ii]]) }
P1_APF0= foreach(ii=1:N, .combine=rbind) %dopar% { APF( mm, Diag=DiagP1[[ii]], k=0 )  }
P1_APF1= foreach(ii=1:N, .combine=rbind) %dopar% { APF( mm, Diag=DiagP1[[ii]], k=1 )  }
DiagP2 =  foreach(ii=1:N) %dopar% { alphaDiag(P2[[ii]]) }
P2_APF0= foreach(ii=1:N, .combine=rbind) %dopar% { APF( mm, Diag=DiagP2[[ii]], k=0 )  }
P2_APF1= foreach(ii=1:N, .combine=rbind) %dopar% { APF( mm, Diag=DiagP2[[ii]], k=1 )  }
DiagP3 =  foreach(ii=1:N) %dopar% { alphaDiag(P3[[ii]]) }
P3_APF0= foreach(ii=1:N, .combine=rbind) %dopar% { APF( mm, Diag=DiagP3[[ii]], k=0 )  }
P3_APF1= foreach(ii=1:N, .combine=rbind) %dopar% { APF( mm, Diag=DiagP3[[ii]], k=1 )  }


fun.curves=rbind(P1_APF0,P2_APF0,P3_APF0)
test=kmeans(fun.curves,centers=3,iter.max = 100,nstart = 200 )
table(test$cluster) # Result of the "kmeans" function


# Code for figure 14 Left panel
par(mar=c(1.5,1.5,0,0))
spatstat.options(transparent=FALSE)
which(test$cluster==2)
plot(P1[[1]],cex=2.5,pch=20,col=1,asp=1,xlim=c(-1.8,2.3),ylim=c(-2.3,1.8),xlab="",ylab="",main="",ann=F,tcl=0,col.axis=0,axes=F)
which(test$cluster==3)
points(P3[[1]],cex=2.5,pch=3,lwd=4,col=4)
which(test$cluster==1)
points(P2[[1]],cex=2.5,pch=2,lwd=4,col=2)
legend("bottomright",c( expression(P[1]~-~Circle~"r=1"),expression(P[2]~-~Two~circles~""),expression(P[3]~-~Circle~"r=0.8") ) ,
       col=c("1","2","4"),y.intersp = 1.5,lwd=c(4,4,6),lty=0,pch=c(20,2,3),cex=2.3,seg.len=2,bty="n")


# Code for figure 14 Middle panel
# Watch for the colors of the curves each time we run the code
par(mar=c(7,7,0.5,0.5))
plot(mm,fun.curves[2,],type="l" ,main="", xlab="" ,ylab="",lwd=0,cex.axis=2.5,xlim=c(0,0.5),ylim=c(0,7) ,mgp=c(3,2,0) )
title(ylab=expression(APF[0](m)), mgp=c(4,1,0) ,cex.lab=3)
title(xlab=expression("m") , mgp=c(5,1,0) ,cex.lab=3)
for (i in which(test$cluster==1)) { lines(mm,fun.curves[i,],col=1,lwd=4) }
for (i in which(test$cluster==2)) { lines(mm,fun.curves[i,],col=2,lwd=4,lty=c("55")) }
for (i in which(test$cluster==3)) { lines(mm,fun.curves[i,],col=4,lwd=6,lty=3 )}
legend("bottomright",c( expression(P[1]~-~Circle~"r=1"),expression(P[2]~-~Two~circles~""),expression(P[3]~-~Circle~"r=0.8") ), 
       col=c("1","2","4"),y.intersp = 1.5,lwd=c(4,4,6),lty=c(1,2,3),cex=2.3,seg.len=2)

# Code for figure 14 Right panel
# Watch for the colors of the curves each time we run the code
par(mar=c(7,7,0.5,0.5))
plot(mm,fun.curves[2,],type="l" ,main="", xlab="" ,ylab="",lwd=0,cex.axis=2.5,xlim=c(0,0.5),ylim=c(0,1.2) ,mgp=c(3,2,0) )
title(ylab=expression(APF[1](m)), mgp=c(4,1,0) ,cex.lab=3)
title(xlab=expression("m") , mgp=c(5,1,0) ,cex.lab=3)
for (i in which(test$cluster==1)) { lines(mm,fun.curves[i,],col=1,lwd=4) }
for (i in which(test$cluster==2)) { lines(mm,fun.curves[i,],col=2,lwd=4,lty=c("55")) }
for (i in which(test$cluster==3)) { lines(mm,fun.curves[i,],col=4,lwd=6,lty=3 )}
legend("topleft",c( expression(P[1]~-~Circle~"r=1"),expression(P[2]~-~Two~circles~""),expression(P[3]~-~Circle~"r=0.8") ), 
       col=c("1","2","4"),y.intersp = 1.5,lwd=c(4,4,6),lty=c(1,2,3),cex=2.3,seg.len=2)

